from pyvirtualdisplay import Display

from django.conf import settings
from django.contrib.auth import get_user, get_user_model
from django.contrib.auth.tokens import default_token_generator
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.core import mail
from django.shortcuts import reverse
from django.template.loader import render_to_string
from django.test import override_settings
from django.utils.encoding import force_text, force_bytes
from django.utils.http import urlsafe_base64_encode, urlsafe_base64_decode
import factory
from freezegun import freeze_time
from rest_framework.test import APITestCase
from rest_framework.exceptions import ErrorDetail
from rest_framework import status
from selenium import webdriver
from selenium.webdriver.common.keys import Keys


from .factories import UserFactory
from .base import TestEmailMixin
from ripple.auth import generate_verify_email_token


class AuthenticationTestData:

    not_registered_user = factory.build(dict, FACTORY_CLASS=UserFactory)
    # test = UserFactory.build()
    @classmethod
    def setUpTestData(cls):

        cls.user = UserFactory()

        super().setUpTestData()


class LoginViewTestCase(AuthenticationTestData, APITestCase):

    end_point = '/api/account/log_in'

    def test_can_login(self):

        user = get_user(self.client)

        self.assertFalse(user.is_authenticated)

        response = self.client.post(self.end_point, {
            'email': self.user.email,
            'password': self.user.password_confirm
        })

        user = get_user(self.client)

        self.assertEqual(response.status_code, 200)
        self.assertTrue(user.is_authenticated)

    def test_cant_login_with_wrong_password(self):

        user = get_user(self.client)

        self.assertFalse(user.is_authenticated)

        response = self.client.post(self.end_point, {
            'email': self.user.email,
            'password': 'wrong'
        })

        user = get_user(self.client)
        self.assertEqual(response.status_code, 400)
        self.assertFalse(user.is_authenticated)

    def test_cant_login_if_user_doesnt_exist(self):

        response = self.client.post(self.end_point, {
            'email': self.not_registered_user['email'],
            'password': 'not_important'
        })

        self.assertEqual(response.status_code, 400)

    # def test_login_validation_if_fields_blank(self):
        # pass


class LogoutViewTestCase(AuthenticationTestData, APITestCase):

    end_point = '/api/account/log_out'

    def test_can_logout(self):

        self.client.force_login(user=self.user)
        user = get_user(self.client)
        self.assertTrue(user.is_authenticated)

        self.client.post(self.end_point)
        user = get_user(self.client)
        self.assertFalse(user.is_authenticated)


class AccountStatusViewTestCase(
        AuthenticationTestData, APITestCase):

    end_point = '/api/account/status'

    def test_can_retreive_account_data(self):

        self.client.force_authenticate(self.user)

        expected = {
            'id': self.user.id,  # TODO red - do we want id on response?
            'email': self.user.email,
            'first_name': self.user.first_name,
            'last_name': self.user.last_name,
            'profile_pic': None,  # TODO red - image field testing
            'is_authenticated': self.user.is_authenticated,
            'is_admin': self.user.is_superuser,
            'is_staff': self.user.is_staff,
            'is_email_verified': self.user.is_email_verified,
            'phone_number': self.user.phone_number
        }

        response = self.client.get(self.end_point)
        
        for key in expected.keys():
            self.assertEqual(response.data[key], expected[key])

    def test_can_retreive_anonymous_data(self):

        expected = {
            'is_email_verified': False,
            'is_staff': False,
            'is_admin': False,
            'is_authenticated': False
        }

        response = self.client.get(self.end_point)

        self.assertDictEqual(expected, response.data)

# TODO red - fix TestEmailMixin


class SignUpTests(AuthenticationTestData):

    def get_valid_post_data(self, extra_kwargs={}):

        return {
            **self.not_registered_user,
            'password': self.not_registered_user['password_confirm'],
            **extra_kwargs
        }

    def get_valid_post_response(self, extra_kwargs={}):
        return self.client.post(
            self.end_point,
            self.get_valid_post_data(extra_kwargs)
        )

    def get_user_response_data(self, response):
        return response.json()

    def test_can_sign_up(self, *args):
        response = self.get_valid_post_response()
        
        self.assertEqual(response.status_code, 201)
        self.assertEqual(
            self.get_user_response_data(response),
            {key: self.not_registered_user[key] for key in [
                'first_name', 'last_name', 'email']}

        )
        user = get_user_model().objects.get(
            email=self.not_registered_user['email'])

    def test_cant_sign_up_with_same_email(self, *args):
        response = self.get_valid_post_response(
            {
                'password': self.not_registered_user['password_confirm'],
                'email': self.user.email,
            }
        )

        self.assertEqual(response.status_code, 400)

        # TODO red look in drf for "(model) with this (field)..." exists
        self.assertDictEqual(
            self.get_user_response_data(response),
            {'email': ['user with this email address already exists.']}
        )

    def test_cant_sign_up_with_bad_email(self, *args):

        response = self.get_valid_post_response({'email': "bad"})

        self.assertEqual(response.status_code, 400)

        self.assertDictEqual(
            self.get_user_response_data(response),
            {'email': ['Enter a valid email address.']}
        )

    def test_cant_sign_up_without_names(self, *args):

        # TODO red - currently failing
        # do we want names enforced?
        response = self.get_valid_post_response({'first_name': ''})

        self.assertEqual(response.status_code, 400)

        # TODO red look in drf for "(model) with this (field)..." exists
        self.assertDictEqual(
            self.get_user_response_data(response),
            {'first_name': ['This field may not be blank.']}
        )

    def test_cant_sign_up_without_matching_passwords(self, *args):
        response = self.get_valid_post_response({'password': 'wrong'})

        self.assertEqual(response.status_code, 400)

        self.assertDictEqual(
            self.get_user_response_data(response),
            {'non_field_errors': ['Password mismatch']}
        )


class SignUpViewTestCase(SignUpTests, APITestCase):

    end_point = '/api/account/sign_up'


class PasswordChangeViewTestCase(AuthenticationTestData, APITestCase):

    end_point = '/api/account/password/change'

    def test_can_change_password(self):

        self.client.force_login(self.user)
        response = self.client.post(
            self.end_point,
            {
                'old_password': self.user.password_confirm,
                'new_password1': 'some_new_thing1',
                'new_password2': 'some_new_thing1',
            }
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.user.refresh_from_db()

        self.assertTrue(self.user.check_password('some_new_thing1'))

    def test_cant_change_with_wrong_old_password(self):

        self.client.force_login(self.user)
        response = self.client.post(
            self.end_point,
            {
                'old_password': 'wrong',
                'new_password1': 'some_new_thing1',
                'new_password2': 'some_new_thing1',
            }
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.user.refresh_from_db()

        self.assertFalse(self.user.check_password('some_new_thing1'))


class PasswordResetRequestViewTestCase(
        AuthenticationTestData, TestEmailMixin, APITestCase):

    end_point = '/api/account/password/reset'

    def test_can_request_reset_email(self):
        response = self.client.post(self.end_point, {
            'email': self.user.email
        })

        uidb64 = urlsafe_base64_encode(force_bytes(self.user.pk)).decode()
        token = default_token_generator.make_token(self.user)

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assert_email_has_been_sent(
            'password_reset_email.txt',
            'password_reset_email.html',
            {
                'email': self.user.email,
                'url': (f"{settings.SITE_URL}/reset-password-confirm"
                        f"?uidb64={uidb64}&token={token}"),
                'user': self.user
            }
        )

    def test_doesnt_send_if_email_not_found(self):

        response = self.client.post(self.end_point, {
            'email': 'not@user.com'
        })

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(mail.outbox), 0)

    def test_error_on_bad_email(self):

        response = self.client.post(self.end_point, {
            'email': 'not_an_email'
        })

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(len(mail.outbox), 0)


class PasswordResetConfirmVewTestCase(AuthenticationTestData, APITestCase):

    end_point = '/api/account/password/reset_confirm'

    def test_can_reset_password(self):

        uidb64 = urlsafe_base64_encode(force_bytes(self.user.pk)).decode()
        token = default_token_generator.make_token(self.user)

        response = self.client.post(
            self.end_point,
            {
                'new_password1': 'something_new_1',
                'new_password2': 'something_new_1',
                'uidb64': uidb64,
                'token': token
            }
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.user.refresh_from_db()

        self.assertTrue(self.user.check_password('something_new_1'))

        # Don't log in after reset
        self.assertFalse(get_user(self.client).is_authenticated)

    def test_invalid_token_bad_request(self):
        uidb64 = urlsafe_base64_encode(force_bytes(self.user.pk)).decode()

        response = self.client.post(
            self.end_point,
            {
                'new_password1': 'something_new_1',
                'new_password2': 'something_new_1',
                'uidb64': uidb64,
                'token': 'tampered_with'
            }
        )

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, {'token': ['Expiried or is invalid']})

        self.user.refresh_from_db()

        self.assertFalse(self.user.check_password('something_new_1'))

    def test_uidb64_for_different_user_bad_request(self):

        other_user = UserFactory()

        other_user_uidb64 = urlsafe_base64_encode(
            force_bytes(other_user.pk)).decode()
        token = default_token_generator.make_token(self.user)

        response = self.client.post(
            self.end_point,
            {
                'new_password1': 'something_new_1',
                'new_password2': 'something_new_1',
                'uidb64': other_user_uidb64,
                'token': token
            }
        )

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, {'token': ['Expiried or is invalid']})
        self.user.refresh_from_db()

        self.assertFalse(self.user.check_password('something_new_1'))


class SendEmailVerificationViewTestCase(
        AuthenticationTestData, TestEmailMixin, APITestCase):

    end_point = '/api/account/send_verify_email'

    def test_can_request_verification_email(self):

        self.client.force_login(self.user)
        token = generate_verify_email_token(self.user.id, self.user.email)

        response = self.client.get(self.end_point)

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assert_email_has_been_sent(
            'verify_email.txt',
            'verify_email.html',
            {
                'url': f"{settings.SITE_URL}/verify?token={token}",
                'user': self.user
            }
        )


class UserUpdateView(AuthenticationTestData, APITestCase):

    def test_can_update_user_data(self):
        self.client.force_login(self.user)
        response = self.client.patch(
            f'/api/account/{self.user.id}/',
            {
                'phone_number': '07777777777'
            },
            format='json'
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data['phone_number'], '07777777777')

    def test_cant_update_someone_elses_details(self):

        user_2 = UserFactory()
        self.client.force_login(self.user)
        response = self.client.patch(
            f'/api/account/{user_2.id}/',
            {
                'phone_number': '07777777777'
            },
            format='json'
        )

        self.assertEqual(response.status_code, 404)

    def test_cant_update_invalid_phone(self):

        user_2 = UserFactory()
        self.client.force_login(self.user)
        response = self.client.patch(
            f'/api/account/{self.user.id}/',
            {
                'phone_number': '777'
            },
            format='json'
        )

        self.assertEqual(response.status_code, 400)


class ConfirmEmailViewTestCase(
        AuthenticationTestData, TestEmailMixin, APITestCase):

    end_point = '/api/account/verify_email'

    def test_can_verify_with_valid_token(self):

        self.assertFalse(self.user.is_email_verified)

        token = generate_verify_email_token(self.user.id, self.user.email)

        response = self.client.post(self.end_point, {'token': token})

        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.user.refresh_from_db()

        self.assertTrue(self.user.is_email_verified)

    def test_can_verify_with_different_email_in_token(self):

        self.assertFalse(self.user.is_email_verified)

        token = generate_verify_email_token(
            self.user.id, 'something_else@test.com')

        response = self.client.post(self.end_point, {'token': token})

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(
            response.json(),
            {
                'token': [('Email address associated with this account does '
                           'not match with confirmation token')]
            }
        )

        self.user.refresh_from_db()

        self.assertFalse(self.user.is_email_verified)



import importlib

from django.conf import settings

def _from_django_settings(settings_name, default):
    import_string = getattr(settings, settings_name)
    if import_string is None:
        import_string = default

    module_path, class_name = import_string.rsplit('.', 1)

    module = importlib.import_module(module_path)
    return getattr(module, class_name)


def get_signup_serializer():
    return _from_django_settings('SIGNUP_SERIALIZER', 'ripple.serializers.SignUpSerializer')


def get_user_serializer():
    return _from_django_settings('USER_SERIALIZER', 'ripple.serializers.UserSerializer')



# Generated by Django 3.0 on 2020-02-27 09:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ripple', '0002_user_phone_number'),
    ]

    operations = [
        migrations.CreateModel(
            name='FeatureOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=200)),
                ('description', models.CharField(blank=True, default='', max_length=200)),
                ('active', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='FeatureUrl',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('description', models.CharField(blank=True, default='', max_length=200)),
                ('regex', models.TextField(max_length=800)),
                ('feature', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='urls', to='ripple.FeatureOption')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from datetime import timedelta
import re
from django.conf import settings

from django.core import signing
from django.core.signing import TimestampSigner
from django.http import HttpResponse, HttpResponseNotFound

from .models import FeatureOption
from django.core.cache import cache


class TesterMiddleware:

    def __init__(self, get_response):
        self.get_response = get_response

        # One-time configuration and initialization.
        # raise MiddlewareNotUsed

    def __call__(self, request):

        # Code to be executed for each request before
        # the view (and later middleware) are called.

        signer = TimestampSigner()
        request.session['features'] = []
        if not request.path.startswith('/admin/'):

            fo = cache.get('featureoptions')

            if fo is None:
                fo = [{'name': f.name, 'urls_re': [re.compile(u.regex) for u in f.urls.all()]}
                      for f in FeatureOption.objects.filter(active=True).all()]
                cache.set('featureoptions', fo, 3600)

            if settings.PRIVATE_URL_RE:
                fo.append({'name': 'site-settings', 'urls_re': [settings.PRIVATE_URL_RE]})

            for f in fo:
                block = True
                if f['name'] in request.COOKIES:
                    value = request.COOKIES[f['name']]
                    try:
                        value = signer.unsign(value, max_age=timedelta(days=2))
                        block = False
                        request.session['features'].append(f['name'])
                    except signing.BadSignature:
                        block = True

                if 'staff' in request.COOKIES:
                    try:
                        signer.unsign(
                            request.COOKIES['staff'], max_age=timedelta(days=2))
                        block = False
                    except signing.BadSignature:
                        block = True

                if block and not request.user.is_staff:
                    for u in f['urls_re']:
                        if u.match(request.path):
                            return HttpResponseNotFound(f'<h1>Not Found {request.path}</h1>')

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        if hasattr(request, 'user') and request.user.is_staff:
            response.set_cookie('staff', signer.sign('staff'), max_age=72000)

        return response



from django.db import models
from django.conf import settings
from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
from django.contrib.auth.models import PermissionsMixin
from django.contrib.auth.tokens import default_token_generator
from django.core.mail import send_mail
from django.core.validators import RegexValidator
from django.template.loader import render_to_string
from django.utils import timezone
from django.utils.encoding import force_bytes, force_text
from django.utils.http import urlsafe_base64_encode
from django.utils.translation import ugettext_lazy as _


from .auth import generate_verify_email_token
from .MailFilter import filtered_send_mail

# remove username from the default django user model. (Creating a custom user model based on the original django model)
# email is used as the unique account 'username'


class UserManager(BaseUserManager):
    use_in_migrations = True

    def _create_user(self, email, password,
                     is_staff, is_superuser, **extra_fields):
        """
        Creates and saves a User with the given username, email and password.
        """
        now = timezone.now()
        email = self.normalize_email(email)
        user = self.model(email=email,
                          is_staff=is_staff, is_active=True,
                          is_superuser=is_superuser,
                          date_joined=now,
                          is_email_verified=False, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email, password=None, **extra_fields):
        return self._create_user(email, password, False, False,
                                 **extra_fields)

    def create_superuser(self, email, password, **extra_fields):
        return self._create_user(email, password, True, True,
                                 **extra_fields)


class User(AbstractBaseUser, PermissionsMixin):

    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=30, blank=True)
    email = models.EmailField(_('email address'), unique=True)
    phone_number = models.CharField(
        null=True,
        max_length=20,
        validators=[
            RegexValidator(
                regex=r'^\+?1?\d{9,15}$',
                message=(_(
                    "Phone number format must be entered in the format: (+)9999999 with optional international code e.g (+4499999). Between 8 and 15 digits allowed."
                ))
            )
        ]
    )
    profile_pic = models.ImageField(blank=True, null=True)
    is_staff = models.BooleanField(_('staff status'), default=False,
                                   help_text=_('Designates whether the user can log into this admin '
                                               'site.'))
    is_active = models.BooleanField(_('active'), default=True,
                                    help_text=_('Designates whether this user should be treated as '
                                                'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    is_email_verified = models.BooleanField(_('email verified'), default=False,
                                            help_text=_('Designates whether email has been verified '))

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    objects = UserManager()

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        "Returns the short name for the user."
        return self.first_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """
        Sends an email to this User.
        """
        send_mail(subject, message, from_email, [self.email], **kwargs)

    def send_verification_email(self):

        token = generate_verify_email_token(self.id, self.email)
        SG_TEMPLATE_ID = "d-f45aba387d824c15a5d6a9b4e97646b7"

        sg_context = {
            'first_name': self.first_name,
            'email': self.email,
            'last_name': self.last_name,
            'verify_url': f"{settings.SITE_URL}/account/verify?token={token}"
        }

        return filtered_send_mail(
            '_',
            '_',
            'info@rippleenergy.com',
            [self.email],
            html_message='_',
            sg_template_id=SG_TEMPLATE_ID,
            sg_context=sg_context
        )

    def send_sign_up_email(self):

        try:

            token = generate_verify_email_token(self.id, self.email)

            template_params = {
                'user': self,
                'url': '%s/account/verify?token=%s' % (settings.SITE_URL, token),
            }

            msg_plain = render_to_string(
                'user_signup_email.txt', template_params)
            msg_html = render_to_string(
                'user_signup_email.html', template_params)

            # TODO how does __rippleweb___ work?

            filtered_send_mail(
                'Welcome to ___rippleweb___',
                msg_plain,
                'support@example.com',
                [self.email],
                html_message=msg_html,
                fail_silently=False,
            )
        except Exception as ex:
            # TODO red log email send failure
            pass

    def send_password_reset_email(self):

        uidb64 = urlsafe_base64_encode(force_bytes(self.pk)).decode()
        token = default_token_generator.make_token(self)

        url = f"{settings.SITE_URL}/account/reset-password-confirm"
        url += f"?uidb64={uidb64}&token={token}"

        SG_TEMPLATE_ID = "d-d56fbf35e1f247ddb7f7f2c372bdea44"

        sg_context = {
            'first_name': self.first_name,
            'url': url
        }

        filtered_send_mail(
            '_',
            '_',
            'support@rippleenergy.com',
            [self.email],
            html_message='_',
            sg_template_id=SG_TEMPLATE_ID,
            sg_context=sg_context
        )

    def send_password_reset_confirmed_email(self):
        # sends a confirmation to someone that their password has been changed
        # TODO red
        pass


class RippleBase(models.Model):

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class AssetBase(RippleBase):

    class Meta:
        abstract = True

    assetid = models.CharField(max_length=200, db_index=True)
    module = models.CharField(
        max_length=200, blank=True, default="", db_index=True)
    version = models.DateTimeField(auto_now=True)
    published = models.BooleanField(default=False)
    owner = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return self.assetid


class TextAsset(AssetBase):

    text = models.CharField(max_length=1000, blank=True, default="")


class RichTextAsset(AssetBase):

    text = models.TextField(blank=True, default="")


class ImageAsset(AssetBase):

    target = models.CharField(max_length=200, default="", db_index=True)
    image = models.ImageField()


class Article(AssetBase):

    layout = models.CharField(
        max_length=200, blank=True, default="", db_index=True)
    image = models.ForeignKey(ImageAsset, on_delete=models.CASCADE)
    text = models.ForeignKey(RichTextAsset, on_delete=models.CASCADE)
    title = models.ForeignKey(TextAsset, on_delete=models.CASCADE)


class ArticleCollection(AssetBase):

    layout = models.CharField(
        max_length=200, blank=True, default="", db_index=True)

    image = models.ForeignKey(ImageAsset, on_delete=models.CASCADE)
    text = models.ForeignKey(RichTextAsset, on_delete=models.CASCADE)
    title = models.ForeignKey(TextAsset, on_delete=models.CASCADE)

    articles = models.ManyToManyField(Article, through='ArticleOrder')
    article_collections = models.ManyToManyField('ArticleCollection', through='ArticleCollectionOrder',
                                                 through_fields=('container_article_collection', 'article_collection'))


class ArticleOrder(models.Model):

    article_collection = models.ForeignKey(
        ArticleCollection, on_delete=models.CASCADE)
    article = models.ForeignKey(Article, on_delete=models.CASCADE)
    order = models.IntegerField()


class ArticleCollectionOrder(models.Model):

    container_article_collection = models.ForeignKey(
        ArticleCollection, related_name="container_articles", on_delete=models.CASCADE)
    article_collection = models.ForeignKey(
        ArticleCollection, related_name="contained_articles", on_delete=models.CASCADE)
    order = models.IntegerField()


class FeatureOption(RippleBase):

    name = models.CharField(max_length=200)
    description = models.CharField(max_length=200, blank=True, default='')
    active = models.BooleanField(default=False)

    def __str__(self):
        return self.name


class FeatureUrl(RippleBase):

    description = models.CharField(max_length=200, blank=True, default='')
    feature = models.ForeignKey(
        FeatureOption, related_name='urls', on_delete=models.CASCADE)
    regex = models.TextField(max_length=800)

    def __str__(self):
        return "{} {}".format(self.feature.name, self.regex[:20])




# Generated by Django 2.1 on 2019-08-09 16:03

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import ripple.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0009_alter_user_last_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('first_name', models.CharField(blank=True, max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=30, verbose_name='last name')),
                ('email', models.EmailField(max_length=254, unique=True, verbose_name='email address')),
                ('profile_pic', models.ImageField(blank=True, null=True, upload_to='')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('is_email_verified', models.BooleanField(default=False, help_text='Designates whether email has been verified ', verbose_name='email verified')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
            },
            managers=[
                ('objects', ripple.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('assetid', models.CharField(db_index=True, max_length=200)),
                ('module', models.CharField(blank=True, db_index=True, default='', max_length=200)),
                ('version', models.DateTimeField(auto_now=True)),
                ('published', models.BooleanField(default=False)),
                ('layout', models.CharField(blank=True, db_index=True, default='', max_length=200)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ArticleCollection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('assetid', models.CharField(db_index=True, max_length=200)),
                ('module', models.CharField(blank=True, db_index=True, default='', max_length=200)),
                ('version', models.DateTimeField(auto_now=True)),
                ('published', models.BooleanField(default=False)),
                ('layout', models.CharField(blank=True, db_index=True, default='', max_length=200)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ArticleCollectionOrder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField()),
                ('article_collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='contained_articles', to='ripple.ArticleCollection')),
                ('container_article_collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='container_articles', to='ripple.ArticleCollection')),
            ],
        ),
        migrations.CreateModel(
            name='ArticleOrder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField()),
                ('article', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ripple.Article')),
                ('article_collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ripple.ArticleCollection')),
            ],
        ),
        migrations.CreateModel(
            name='ImageAsset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('assetid', models.CharField(db_index=True, max_length=200)),
                ('module', models.CharField(blank=True, db_index=True, default='', max_length=200)),
                ('version', models.DateTimeField(auto_now=True)),
                ('published', models.BooleanField(default=False)),
                ('target', models.CharField(db_index=True, default='', max_length=200)),
                ('image', models.ImageField(upload_to='')),
                ('owner', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RichTextAsset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('assetid', models.CharField(db_index=True, max_length=200)),
                ('module', models.CharField(blank=True, db_index=True, default='', max_length=200)),
                ('version', models.DateTimeField(auto_now=True)),
                ('published', models.BooleanField(default=False)),
                ('text', models.TextField(blank=True, default='')),
                ('owner', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TextAsset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('assetid', models.CharField(db_index=True, max_length=200)),
                ('module', models.CharField(blank=True, db_index=True, default='', max_length=200)),
                ('version', models.DateTimeField(auto_now=True)),
                ('published', models.BooleanField(default=False)),
                ('text', models.CharField(blank=True, default='', max_length=1000)),
                ('owner', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='articlecollection',
            name='article_collections',
            field=models.ManyToManyField(through='ripple.ArticleCollectionOrder', to='ripple.ArticleCollection'),
        ),
        migrations.AddField(
            model_name='articlecollection',
            name='articles',
            field=models.ManyToManyField(through='ripple.ArticleOrder', to='ripple.Article'),
        ),
        migrations.AddField(
            model_name='articlecollection',
            name='image',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ripple.ImageAsset'),
        ),
        migrations.AddField(
            model_name='articlecollection',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='articlecollection',
            name='text',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ripple.RichTextAsset'),
        ),
        migrations.AddField(
            model_name='articlecollection',
            name='title',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ripple.TextAsset'),
        ),
        migrations.AddField(
            model_name='article',
            name='image',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ripple.ImageAsset'),
        ),
        migrations.AddField(
            model_name='article',
            name='owner',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='article',
            name='text',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ripple.RichTextAsset'),
        ),
        migrations.AddField(
            model_name='article',
            name='title',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ripple.TextAsset'),
        ),
    ]



from django.core.management.base import BaseCommand, CommandError

class Command(BaseCommand):
    help = "Creating Ripple Project"

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **options):
        print("Creating Ripple Project")